open Atypes
open Printf

let verbose = ref false
let host = ref (Unix.gethostname())
let nclients = ref 1
and nbursts = ref 0

let () =
  Arg.parse
    [
     "-v", Arg.Set verbose, " be verbose" ;
     "-nbursts", Arg.Int (fun i -> nbursts := i),
      (sprintf "number of local bursts [%i]" !nbursts);
     "-nclients", Arg.Int (fun i -> nclients := i),
      (sprintf "number of local clients in burst [%i]" !nclients);
    ] 
    (fun h -> host := h)
    (sprintf "usage: %s [opt]*" Sys.argv.(0))

(* let debug id msg = Join.debug0 id msg *)

let players_to_string ps =
  String.concat "; "  (List.map string_of_int ps)

type status =
  { id : int ;  tick : unit channel ;
    best_vote : vote ;
    next : int ; next_when_vote : int ; }

let pr_vote v =
  sprintf "{turn=%i; rand=%i; vid=%i;}" v.turn v.rand v.vid

let vote_create st =
  let r = 
  { turn = st.best_vote.turn + 1 ;
    rand = Random.int 1000 ;
    vid  = st.id ; } in
(*  debug "CREATE" (sprintf "[%i] %s" st.id (pr_vote r)) ; *)
  r


let first_vote id = { turn = -1 ; rand = Random.int 1000 ; vid=id ; }

let vote_compare v1 v2 = Pervasives.compare v1 v2

let client(register, kill_me) =

  def left(Vote x) & myright(hisleft)  & status(st) = 
    Join.debug "VOTE"
      "[%i]: %s over %s" st.id (pr_vote x) (pr_vote st.best_vote) ;
    myright(hisleft) & 
    begin
      let res = vote_compare x st.best_vote in
      if res > 0 then begin
         hisleft(Vote x) &
         status({st with best_vote=x ; next_when_vote=st.next ;})
      end else if res=0 then begin
        Join.debug "LEADER" "id=%i, vote=%s" st.id (pr_vote x) ;
        status({st with next_when_vote=st.next}) &
        begin
          hisleft(Play (st.id, hisleft, []))
        end
      end else begin
       status(st) & hisleft(Vote st.best_vote)
      end
    end

  or left(Play (leader, ll, players)) & myright(hisleft) & status(st)  =
(*    debug "PLAY" (sprintf "[%i]" st.id) ; *)
     status(st) & save(hisleft) &
     if st.next_when_vote = st.next then begin
       if st.id = leader then begin
         def k() =
           Join.debug "GAME" "%s" (players_to_string (leader::players)) ;
           Thread.delay 5.0 ;
           Join.debug "GAME OVER" "[%i]" st.id ;
           st.tick() & hisleft(Restart leader) in
         hisleft(Rechange (leader,k))
       end else
         hisleft (Play (leader, ll, st.id :: players))
     end else begin
       Join.debug "MALAISE"
         "[%i] next changed %i -> %i"
            st.id st.next_when_vote st.next ;
       ll(Failed st.id)
     end

  or left(Rechange (leader,k) as msg) & save(hisleft) & status(st) =
(*       debug "RECHANGE" (sprintf "[%i]" st.id) ; *)
       myright(hisleft) & status(st) &
       if st.id <> leader then hisleft(msg)
       else k()

  or left(Restart leader) & myright(hisleft) & status(st) =
       Join.debug "RESTART" "[%i]" st.id ;
       let new_vote = vote_create st in
       myright(hisleft) & status({ st with best_vote = new_vote }) &
       if leader <> st.id then  hisleft(Restart leader)
       else hisleft(Vote new_vote)

  or left(Failed (leader) as msg) & save(hisleft) & status(st) =
       Join.debug "FAILED" "[%i]" st.id ;
       let new_vote = vote_create st in
       myright(hisleft) & status({ st with best_vote = new_vote }) &
       if leader <> st.id then hisleft(msg)
       else hisleft(Vote new_vote)

  or myright (_) & change(newleft,m,k) & status(st) =
     Join.debug "CHANGE" "[%i] %i -> %i" st.id st.next m ;
     myright(newleft) & status({st with next=m}) & k()

  in

  let (n,tick,((id0,idn),(left0, change_n))),k =
    register ((left, change),kill_me) in

  Join.debug "START" "[%i]: (%i, %i)" n id0 idn ;


  def when_changed() =
    Join.debug "CHANGED" "%i" n ;
    k() in

  let vote = first_vote n in
  let st =
    { id = n ; tick = tick ;
      best_vote = vote ; next=id0 ; 
      next_when_vote = id0 ; } in
  spawn begin
     myright(left0) & change_n(left,n, when_changed)
     & status(st) &
     if n=id0 then left0(Vote vote)
  end ;
  ()

open Unix

let server = !host

let addr = (gethostbyname server).h_addr_list.(0)

let ns = Ns.register_client addr 12345

let register_me = (Ns.lookup ns "register_me" : register )

def kill_me () & wait () = reply to kill_me & reply to wait


let _ =
  spawn begin
    client(register_me, Some kill_me) ;
    for j = 1 to !nbursts do
      Thread.delay(5.0) ;
      for n = 1 to !nclients do
        client(register_me, None)
      done
    done ;
    0
  end

let _ = wait() ; Join.flush_space ()
