open Atypes
open Printf

let ngames = ref 10
and verbose = ref false

let () =
  Arg.parse
    ["-v", Arg.Set verbose, " be verbose"]
    (fun s -> ngames := int_of_string s)
    (sprintf "usage: %s [opt]* [nplays]" Sys.argv.(0))

let debug id msg = if !verbose then Join.debug0 id msg

let () = Join.listen (Unix.ADDR_INET (Join.local_addr, 12345))

let ns = Ns.local


def over() & wait() = reply to wait
or  kill() & wait() = reply to wait
or  remaining(n) & tick() =
  Join.debug "TICK" "%i" n ;
  if n = 1 then over() else remaining(n-1)

let _ = spawn remaining(!ngames)

let to_kill = ref []

let add_kill = function
  | Some f -> to_kill := f :: !to_kill
  | None -> ()

def bidon (_) = 0

let table_create me = Table.create (me)

def register_me(me,kill_me) & empty() =
  add_kill(kill_me) ;
  let t = table_create (0,me) in
  def k() = full(t) in
  reply (0,tick,((0,0),me)), k to register_me

or register_me(me,kill_me) & full(t) =
  add_kill(kill_me) ;
  let n = Table.size t in
  let (id0,(left0,_)), (idn,(_,ch_n)) = Table.insert t (n, me) in
  def k() = full(t) in
  reply (n, tick, ((id0,idn),(left0, ch_n))),k to register_me


let _ = spawn empty()

let _ = Ns.register ns "register_me" register_me


let _ =
  wait () ;
  List.iter (fun f -> f ()) !to_kill
