open Unix
(* Reciprocal connection, explicit inet identitity *)
(*OUT
0: 1
1: 0
*)

(*ERR
Exit
Exit
*)

(*DIFF OUT ./diffsorted *)

let server = ref (gethostname())
let son = ref false

let () =
  Arg.parse
    ["-son", Arg.Set son, "adopt son behavior"]
    (fun s -> server := s)
    (Printf.sprintf "%s [opt]* [server]" Sys.argv.(0))

let server_addr = (gethostbyname !server).h_addr_list.(0)

let zyva id oid =

  let identity = 
    Unix.ADDR_INET (Join.local_addr, if id = "0" then 12346 else 12347) in
  Join.listen identity ;

  let ns = Ns.connect (ADDR_INET (server_addr, 12345)) in

  let rec hard_get name =
    try Ns.lookup ns name with
    | Not_found -> Thread.delay 1.0 ; hard_get name in

  let server = (hard_get "server" : Join.site) in
  
  def wait () & lock() = reply to wait in

  let () = Join.at_fail server
      (def e() = prerr_endline "Exit" ; lock() in e) in

  let tick = (hard_get "tick" : unit channel) in

  def work(x) =
    Printf.printf "%s: %i\n" id x ; flush Pervasives.stdout ; tick() in

  let () = Ns.register ns (id ^ "work") work in

  
  let owork = (hard_get (oid ^ "work") : int channel) in

  let sync = (hard_get ("sync"^ id) : unit -> unit) in

(* Synchronize with other client *)
  let () = sync () in

(* This favors simultaneous send below *)

  let () = spawn (owork(int_of_string id)) in

  wait()

let _ =
  if not !son then  match fork () with
  | 0 ->
      execv Sys.argv.(0) [| Sys.argv.(0) ; "-son" ; !server |]
  | _ ->
      zyva "1" "0" ; ignore (wait()) ; exit 0
  else begin
    zyva "0" "1" ;
    exit 0
  end
