{
open Printf
open Arg

let name = ref None

let usage = sprintf "Usage: %s test_name" Sys.argv.(0)

let _ =
  parse []
  (fun s -> name := Some s)
  usage

let name = match !name with
| Some r -> r
| None -> prerr_endline usage ; exit 2

let base = Filename.chop_extension name

let out_chan = open_out (base^".out")
and err_chan = open_out (base^".err")

let differr = ref "diff"
and diffout = ref "diff"

}

rule main = parse
| eof  { () }
| "(*OUT" [^'\n']* '\n' { to_chan out_chan lexbuf ; main lexbuf }
| "(*ERR" [^'\n']* '\n' { to_chan err_chan lexbuf ; main lexbuf }
| "(*DIFF" [' ']* "ERR" [' ']+ (['.''/''a'-'z']+ as cmd) [^'\n']* '\n'
   {differr := cmd ; main lexbuf }
| "(*DIFF" [' ']* "OUT" [' ']+ (['.''/''a'-'z']+ as cmd) [^'\n']* '\n'
   {diffout := cmd ; main lexbuf }
|  [^'\n']* '\n'        { main lexbuf }
| "" { failwith "main" }

and to_chan chan = parse
| "*)" [^'\n']* '\n'    { () }
|  [^'\n']* '\n' as line
    { output_string chan line ; to_chan chan lexbuf }
| "" { failwith "to_chan" }

{
let in_chan = open_in name

let _ = main (Lexing.from_channel in_chan)

let _ = close_in in_chan ; close_out out_chan ; close_out err_chan 

let save_diff ext cmd =
  let outc = open_out (base^"diff."^ext) in
  output_string outc cmd ;
  close_out outc

let _ = save_diff "err" !differr ; save_diff "out" !diffout

let _ = exit 0
}
