(*
  Various loops
*)

(* Asynchronous, with waiter *)
(*OUT
*****
*)

let waiter n = 
  def count(0) & wait() = reply to wait
  or  count(n) & tick() = count(n-1) in
  spawn count(n) ;
  tick, wait

def loop(a,x) =
  if x > 0 then begin
    a() & loop (a,x-1)
  end

let _ =
  let n = 5 in
  let tick, wait = waiter n in
  def echo_star() = print_char '*' ; tick() in
  spawn loop(echo_star, n) ;
  wait() ;
  print_newline ()

(* Asynchronous, with for loop *)

(*OUT
*****
*)

def loop(f,n) =  for i = 1 to n do f() done 

let _ =
  let n = 5 in
  let tick, wait = waiter n in
  def echo_star() = print_char '*' ; tick() in
  spawn loop(echo_star, n) ;
  wait() ;
  print_newline ()

(* Synchronous *)

(*OUT
54321
*)

def loop(a,x) =
  if x > 0 then begin
    a(x) ; reply loop(a,x-1) to loop
  end else
    reply to loop

def print(x) = print_int x ; reply to print

let _ = loop(print, 5) ; print_newline ()

(* Accumulating loop *)

(*OUT
11440
*)
let sum i0 f =

  def add(dr) & total(r,i) =
    let r' = r+dr in
    if i > 1 then reply total(r',i-1) to total
    else  reply r' to total in

  def loop(i) =
    if i > 0 then begin
      add(f i) & loop(i-1)
    end in

  spawn loop(i0) ;
  total(0,i0)

let _ =
  print_int (sum 32 (def square(x) = reply x*x to square in square)) ;
  print_newline ()

(* Distributed loop *)

(*OUT
11440
*)
let square1 i = i*i

let square2 i =
  def total(r) & wait() = reply r to wait in
  def mult(r,j) = if j <= 0 then total(r) else mult(r+i,j-1) in
  spawn mult(0,i) ;
  wait()

let make_sum i0 =

  def add(dr) & total(i) =
    if i > 1 then reply dr+total(i-1) to total
    else reply dr to total in

  let wait () = total(i0) in

  def loop(i) & register(f) =
    if i > 0 then begin
      loop(i-1) &
      let dr = f i in
      begin add (dr) & register(f)  end
    end in

  spawn loop(i0) ;
  register, wait

let register, wait = make_sum 32

let _ = spawn begin register(square1) & register(square2) end

let _ = print_int (wait ()) ; print_newline ()
